#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/* 
 * circle.c
 *
 * FUNCTION:
 * generate pixmaps showing Poincare recurrence time for circle map.
 *
 * HISTORY:
 * created Linas Vepstas Jan 1991
 * spruced up Linas Vepstas July 1993

XXXXXXXXX  ----------- OBSOLETE ------- XXXXXXXXXXXX use image.c for modern work
 */

/*-------------------------------------------------------------------*/
/* Test routine, draws very pretty moire pattern */

void moire_parabola (float *glob, unsigned int sizex, unsigned int sizey)
{
   unsigned int	i,j, globlen;
   int		icen, jcen, ip, jp;
   float	globby;
   
   globlen = sizex*sizey;
   for (i=0; i<globlen; i++) glob [i] = 0.0;
   
   icen = 0.65* sizex;
   jcen =  0.8 * sizey;
   for (i=0; i<sizey; i++)
      {
      for (j=0; j<sizex; j++) 
         {
         ip = i - icen; 
         jp = j - jcen;
         globby = (float) ((ip*ip+jp*jp) % 25000);
         globby = globby / 25000.0;
         glob [i*sizex +j] = globby; 
         }
      }
}

#define PI 3.14159265358979

/*-------------------------------------------------------------------*/
/* 
 * This routine computes average value taken by
 * circle map iterator.
 */

void circle_map (float *glob, 
                 unsigned int sizex,  /* width, in pixels */
                 unsigned int sizey,  /* height, in pixels */
		 double omega_min, 
                 double omega_max, 
                 double K_min, 
                 double K_max, 
                 int itermax)         /* max number of iterations */
{
   unsigned int	i,j, globlen;
   float	delta_omega, delta_K;
   float	omega, K;
   float	x;
   int		iter;
   
   
   /* initialize pixmap to zero */
   globlen = sizex*sizey;
   for (i=0; i<globlen; i++) glob [i] = 0.0;
   
   /* compute slopes */
   delta_omega = (omega_max - omega_min) / ((float) sizex);
   delta_K = (K_max - K_min) / ((float) sizey);
   
   /* loop over columns, then rows */
   for (i=0; i<sizey; i++) { 
      printf (" starting %d out of total %d \n", i, sizey);
      K = (((float) i) +0.5) * delta_K + K_min ;
      for (j=0; j<sizex; j++) {
         omega = (((float) j) +0.5) * delta_omega + omega_min;
   
         /* OK, now start iterating the circle map */
         x = 0.0;
         for (iter=0; iter<itermax; iter++) {
            x += omega - K * sin (2.0 * PI * x);
         }
         /* x is the normalized winding number */
         x /= (float) itermax;
         glob [i*sizex +j] = x;    /* record the data */
   /*==============
         if (x<0.0) x = 0.0;
         if (x>1.0) x = 1.0;
         if ((x<0.0) | (x>1.0)) {
            printf ("out of bounds omega=%f and K=%f W=%f \n", omega, K, x);
         }
         printf ("for omega=%f and K=%f W=%f \n", omega, K, x);
   ==============*/
      }
   }
}
   
/*-------------------------------------------------------------------*/

#define EPSILON  	0.02
#define SETTLE_TIME 	500
#define NORMALIZATION 	30.0

/*-------------------------------------------------------------------*/
/*
 * Compute the poincare recurrance time for circle map.
 */

void circle_recurrance (float *glob, 
                 unsigned int sizex,  /* width, in pixels */
                 unsigned int sizey,  /* height, in pixels */
		 double omega_min, 
                 double omega_max, 
                 double K_min, 
                 double K_max, 
                 int itermax)         /* max number of iterations */
{
   unsigned int	i,j, globlen;
   float	delta_omega, delta_K;
   float	omega, K;
   float	x;
   float	xpoint;
   int		iter;
   long		num_recurs, time_recur;
   
   
   /* initialize pixmap to zero */
   globlen = sizex*sizey;
   for (i=0; i<globlen; i++) glob [i] = 0.0;
   
   /* compute slopes */
   delta_omega = (omega_max - omega_min) / ((float) sizex);
   delta_K = (K_max - K_min) / ((float) sizey);
   
   /* loop over columns, then rows */
   for (i=0; i<sizey; i++) { 
      printf (" starting %d out of total %d \n", i, sizey);
      K = (((float) i) +0.5) * delta_K + K_min; 
      for (j=0; j<sizex; j++) {
         omega = (((float) j) +0.5) * delta_omega + omega_min;
   
         /* OK, now start iterating the circle map */
         /* First, we give a spin for 500 cycles, giving the non-chaotic 
          * parts a chance to phase-lock */
         x = 0.0;
         for (iter=0; iter<SETTLE_TIME; iter++) {
            x += omega - K * sin (2.0 * PI * x);
         }
  
         /* OK, now, we begin to measure the average amount of time to recur */
         xpoint = x;
         num_recurs = 0;
         for (iter=0; iter<itermax; iter++) {
            x += omega - K * sin (2.0 * PI * x);
            if (fabs(fmod ((x-xpoint), 1.0)) < EPSILON) {
               num_recurs ++;
               time_recur = iter;
            }
         }
         /* x is the (normalized) number of cycles to reach recurrance */
         x = (float) time_recur / (NORMALIZATION * ((float)num_recurs));
         glob [i*sizex +j] = x;    /* record the data */
      }
   }
}
   
/*-------------------------------------------------------------------*/
   
extern FILE *Fopen();

void do_circle (char * filename, int width, int height)

{
   float	*data ;		/* my data array */
   unsigned int	data_width, data_height;/* data array dimensions */
   int		itermax;
   float		omega_min, omega_max, K_min, K_max;
   FILE		*fp;

   data = (float *) malloc (sizeof (float) * width * height);
   if (data == NULL) {
      fprintf (stderr, "Whoops, no memory \n");
      exit (EXIT_FAILURE);
   }

   data_width = width;
   data_height = height;
   /*   The BIG picture
   */
   omega_min = 0.0;
   omega_max = 1.0;
   K_min = 1.0;
   K_max = 0.0;
   /* Close up of central flame */
/*
   omega_min = 5.0/12.0;
   omega_max = 7.0/12.0;
   K_min = 1.0/3.0;
   K_max = 1.0/6.0;
*/
   
   itermax = 2000;
   (void) circle_recurrance (data, data_width, data_height, 
                  omega_min, omega_max, K_min, K_max, itermax);
   
   /* dump the floating point data */
   if ( (fp = Fopen (filename, ".flo")) == NULL) {
      fprintf (stderr, " File open failure \n");
      exit (EXIT_FAILURE);
   }

   fprintf (fp, "%d %d\n", data_width, data_height);
   fwrite (data, sizeof(float), data_width*data_height, fp);
   fclose (fp);
   
   if ( (fp = Fopen (filename, ".txt")) == NULL) {
      fprintf (stderr, " File open failure \n");
      exit (EXIT_FAILURE);
   }

   fprintf (fp, "standard circle map\n");
   fprintf (fp, "x(n+1) = x(n) + Omega - K * sin(2*PI*x(n))\n");
   fprintf (fp, "Each pixel shows number of cycles to recur within EPSILON \n");
   fprintf (fp, "where EPSILON = %f \n", EPSILON);
   fprintf (fp, "But first, system allowed to settle for %d cycles\n", SETTLE_TIME);
   fprintf (fp, "Recurrance time normalized by %f\n", NORMALIZATION);
   fprintf (fp, "\n");
   fprintf (fp, "floating point values, one per pixel, (0<x<1); \n");
   fprintf (fp, "Omega on X axis min = %f max = %f \n", omega_min, omega_max);
   fprintf (fp, "K on Y axis min = %f max = %f \n", K_max, K_min);
   fprintf (fp, "width: %d height: %d \n", data_width, data_height);
   fprintf (fp, "Iteration stopped after %d iterations \n", itermax);
   fclose (fp);
   
}

/*-------------------------------------------------------------------*/

void main (int argc, char *argv[]) 

{

   if (argc != 4) {
      fprintf (stderr, "Usage: %s <out-file> <width> <height> \n", argv[0]);
      exit (EXIT_FAILURE);
   }

   do_circle (argv[1], atoi (argv[2]), atoi (argv[3]));

   exit (EXIT_SUCCESS);
}
